﻿define([
    'jquery',
    'nico/drop',
    'nico/url',
    './afbeelding',
    'nico/Designer/propertyEditors'
], function ($, Drop, url, Afbeelding, propertyEditors) {
    'use strict';

    var AfbeeldingDesign = function (options, $element) {
        Afbeelding.call(this, options, $element);

        this.storedFilename = this.data.filename;

        if (this.data.filenameInherited)
            this.$element
                .addClass('inheritedTekst');

        this.initAnchorData(this.data);
        this.addClicksForEdit(this.getPropertyEditors());
    };

    Drop.extend(AfbeeldingDesign, Afbeelding);

    AfbeeldingDesign.prototype.getPropertyEditors = function () {
        var self = this;

        return [
            propertyEditors.createFile('File')
                .withGet(function () {
                    return self.data.filename;
                })
                .withSet(function (value) {
                    if (self.storedFilename === value) {
                        self.data.filename = value;

                        if (value)
                            self.setImageSrc(url.toUploadUrl(value));
                        else
                            self.removeImage();
                    } else {
                        self.data.filename = value;

                        if (value)
                            self.setImageSrc(url.toTempUrl(value));
                        else
                            self.removeImage();
                    }
                })
                .withDataDecorator('filename')
        ];
    };

    AfbeeldingDesign.prototype.addSaveData = function (data) {
        data.filename = this.data.filename;
    };

    AfbeeldingDesign.prototype.afterSave = function () {
        this.storedFilename = this.data.filename;
        if (this.storedFilename)
            this.setImageSrc(url.toUploadUrl(this.storedFilename));
        else
            this.removeImage();
        this.$element
            .removeClass('inheritedTekst');
    };

    AfbeeldingDesign.prototype.createOperationRoute = "addAfbeeldingDrop";
    AfbeeldingDesign.prototype.updateOperationRoute = "updateAfbeeldingDrop";
    AfbeeldingDesign.prototype.deleteOperationRoute = "deleteAfbeeldingDrop";

    return AfbeeldingDesign;
});