﻿define([
    'jquery',
    'underscore',
    'select2',
    'nico/dispatcher'
], function ($, _, select2, Dispatcher) {
    'use strict';

    var CardEditor = function ($element) {
        this.dispatcher = new Dispatcher();
        this.$card = $('<div />')
            .css({
                width: '100%',
                height: '100%'
            })
            .appendTo($element);

        var self = this;
        this.$card
            .select2({
                minimumInputLength: 2,
                multiple: false,
                query: function(query) {
                    self.dispatcher
                        .post('getFilteredCards', { filter: query.term })
                        .done(function(data) {
                            var results = data.results;

                            query.callback({
                                results: self.convertStringsToListItems(results)
                            });
                        });
                }
            });

        $element
            .find('input.select2-input')
            .keydown(function (e) {
                return e.keyCode !== 32;
            })
            .bind('cut copy paste', function (e) {
                e.preventDefault();
            });
    };

    CardEditor.prototype.onChanged = function (eventHandler) {
        var self = this;
        this.$card.change(function (e) {
            var data = self.$card.select2('data');
            if (data) {
                var cardReference = { id: data.id, naam: data.text };
                eventHandler(cardReference);
            }
        });
    };


    CardEditor.prototype.convertStringsToListItems = function (cards) {
        return _.map(cards, function (result) {
            if(result.id)
                return { id: result.id, text: result.naam };
            else 
                return { id: result, text: result };
        });
    };

    CardEditor.prototype.setCard = function (card) {
        this.$card.select2('data', { id: card.id, text: card.naam });
    };

    CardEditor.prototype.getCard = function () {
        return this.$card.select2("data"); 
    };

    return CardEditor;
});