﻿define([
    'jquery',
    'nico/drop',
    'owlCarousel'
], function ($, Drop, owlCarousel) {
    'use strict';

    var defaults = function () {
        return {
            volgende: 'Volgende',
            vorige: 'Vorige'
        };
    };

    function CarouselPager(options, $element) {
        Drop.call(this, $element, defaults());

        this.data = $.extend(this.data, options);

        this.renderedPromise = $.Deferred();

        this.$vorige = $('<span />')
            .text(this.data.vorige);

        this.$volgende = $('<span />')
            .text(this.data.volgende);

        var self = this;
        this.$element

            // note: the order in which the links are added is important for a correct layout
            .append($('<a />')
                .addClass('next button with-icon small green')
                .append(this.$volgende)
                .append($('<span />').addClass('icon icon-arrow-right'))
                .click(function() {
                    self.next();
                }))
            .append($('<a />')
                .addClass('prev button with-icon small green')
                .append($('<span />').addClass('icon icon-arrow-left'))
                .append(this.$vorige)
                .click(function() {
                    self.previous();
                }));
    };

    Drop.extend(CarouselPager, Drop);

    CarouselPager.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    CarouselPager.prototype.preRender = function () {
        this.container = this.lookupContainer();

        var self = this;
        this.container.promise().done(function () {
            if (self.container.pool.length !== 0) {
                self.container.$element.owlCarousel({
                    singleItem: true,
                    slideSpeed: 300
                });
            }

            if (self.container.pool.length > 1)
                self.$element.show();
            else
                self.$element.hide();

            self.renderedPromise.resolve();
        });
    };

    CarouselPager.prototype.lookupContainer = function() {
        return _.find(this.parent.pool, function (drop) {
            return drop.type === 'dropContainer' && drop.$element.hasClass('owl-carousel');
        });
    };

    CarouselPager.prototype.next = function () {
        this.container.$element.trigger('owl.next');
    };

    CarouselPager.prototype.previous = function () {
        this.container.$element.trigger('owl.prev');
    };

    return CarouselPager;
});