﻿define([
    'jquery',
    'nico/drop',
    'nico/dropFactory'
], function ($, Drop, dropFactory) {
    'use strict';

    function DropContainer(options, $template) {
        Drop.call(this, $template.parent(), options);

        this.renderedPromise = $.Deferred();
        this.$template = $template
            .attr('data-ignore', 'true')
            .detach();
    };

    Drop.extend(DropContainer, Drop);

    DropContainer.prototype.render = function () {
        this.restoreDrops();
    };

    DropContainer.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    DropContainer.prototype.restoreDrops = function () {
        if (!this.data.drops || this.data.drops.length === 0) {
            this.renderedPromise.resolve();
        } else {
            var self = this;
            _.each(this.data.drops, function (drop) {
                drop.dropRuntimeData.quizMaster = self.data.quizMaster;
                drop.dropRuntimeData.conditional = self.data.conditional;
            });

            dropFactory
               .restoreDrops(this.environment, 'buildingBlocks', this.data.drops, this.$template)
               .done(function (drops) {
                   _.each(drops, function (drop) {
                       self.addDrop(drop);
                       self.$element.append(drop.$element);
                   });

                   var promises = [];
                   _.each(drops, function (drop) {
                       drop.render();

                       if (_.isFunction(drop.promise))
                           promises.push(drop.promise());
                   });

                   if (promises.length === 0) {
                       self.renderedPromise.resolve();
                   } else {
                       $.when
                           .apply($, promises)
                           .done(function () {
                               self.renderedPromise.resolve();
                           });
                   }
               });
        }

    };

    return DropContainer;
});