﻿define([
    'jquery',
    'nico/drop',
    './dropContainerEditor',
    'nico/dropFactory',
    'nico/observable',
    'nico/Designer/dropPersistence'
], function ($, Drop, DropContainerEditor, dropFactory, observable, dropPersistence) {
    'use strict';

    function DropContainerDesign(options, $template) {
        if ($template.parent().length === 0)
            throw new Error('You can only add "data-drop-many" to nodes that have a parent node in the template');

        Drop.call(this, $template.parent(), options);
        observable.makeObservable(this);

        this.$template = $template
            .attr('data-ignore', 'true')
            .detach();

        this.renderedPromise = $.Deferred();

        var self = this;
        this.addDataDecorator(function(data) {
            data.nestedDropType = self.data.nestedDropType;
        });
        this.addClicksForEdit(this.getPropertyEditors());
    };

    Drop.extend(DropContainerDesign, Drop);

    DropContainerDesign.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    DropContainerDesign.prototype.render = function () {
        this.restoreDrops();
    };

    DropContainerDesign.prototype.restoreDrops = function () {
        if (!this.data.drops || this.data.drops.length === 0) {
            this.renderedPromise.resolve();
        } else {
            var self = this;
            dropFactory
                .restoreDrops(this.environment, 'buildingBlocks', this.data.drops, this.$template)
                .done(function (drops) {
                    self.prepareDrops(drops);
                    self.renderDrops(drops);
                    self.renderedPromise.resolve();
                });
        }
    };

    DropContainerDesign.prototype.prepareDrops = function (drops) {
        var self = this;
        _.each(drops, function (drop) {
            drop.data.conditional = self.data.conditional;

            self.addDrop(drop);

            drop.addSaveDataDecorator(function (data) {
                self.decorateSaveData(data);
                data.path.push(self.id);
            });

            self.$element.append(drop.$element);
        });
    };

    DropContainerDesign.prototype.renderDrops = function(drops) {
        _.each(drops, function (drop) {
            drop.render();
        });
    };

    DropContainerDesign.prototype.insertDrop = function () {
        var self = this;
        var $targetElement = this.createEmptyNode();
        dropFactory
            .createDrop(this.environment, 'buildingBlocks', this.data.nestedDropType, {
                positioning: 'relative',
                conditional: self.data.conditional
            }, $targetElement)
            .done(function (drop) {
                self.addDrop(drop);

                drop.addSaveDataDecorator(function (data) {
                    self.decorateSaveData(data);
                    data.path.push(self.id);
                });

                if (drop.type === 'widget') {
                    dropPersistence.saveDrop(drop);
                    drop.promise().done(function() {
                        self.informListeners('dropAdded', drop);
                    });
                } else {
                    self.informListeners('dropAdded', drop);
                }

                drop.render();

                self.$element
                    .append($targetElement);
            });
    };

    DropContainerDesign.prototype.createEmptyNode = function () {
        return this.$template.clone();
    };

    DropContainerDesign.prototype.deleteDrop = function (drop) {
        if (drop.id)
            dropPersistence.remove(drop);

        drop.$element.remove();

        this.removeDrop(drop);
        this.informListeners('dropDeleted', drop);
    };

    DropContainerDesign.prototype.getPropertyEditors = function () {
        return [
            new DropContainerEditor(this)
        ];
    };

    DropContainerDesign.prototype.createOperationRoute = 'addDropContainer';

    return DropContainerDesign;
});