﻿define([
    'jquery',
    'underscore',
    'nico/Designer/PropertyEditors/propertyEditor'
], function ($, _, PropertyEditor) {
    'use strict';

    function DropContainerEditor(dropList) {
        PropertyEditor.call(this);
        this.dropList = dropList;

        var self = this;
        this.dropList.addListener('dropAdded', function(drop) {
            self.addPropertiesOfDrop(drop);
        });

        this.withGet($.noop);
        this.withSet($.noop);
    }

    PropertyEditor.extend(DropContainerEditor, PropertyEditor);

    DropContainerEditor.propertyEditorName = 'DropListEditor';

    DropContainerEditor.prototype.render = function ($body) {
        this.$drops = $('<div />');

        this.$add = $('<button />')
            .text('Add ' + this.dropList.data.nestedDropType)
            .addClass('button small green')
            .click(function () {
                self.dropList.insertDrop();
            });

        this.$editor = $('<div />')
            .append(this.$drops, this.$add);

        var $row = this.createRow($body, this.label, this.$editor);

        var self = this;
        _.each(this.dropList.pool, function (drop) {
            self.addPropertiesOfDrop(drop);
        });

        return $row;
    };

    DropContainerEditor.prototype.addPropertiesOfDrop = function (drop) {
        var $drop = $('<div />')
            .appendTo(this.$drops);

        var self = this;
        $('<button />')
            .text('Delete')
            .addClass('button small green')
            .click(function () {
                if (!confirm('Ben je zeker dat je deze drop wil verwijderen?'))
                    return;

                self.dropList.deleteDrop(drop);
                $drop.remove();
            })
            .appendTo($drop);

        this.callDropPropertyEditors(drop, function (e) {
            e.prepare();
            e.render($drop);
        });

        if (drop.getNestedDrops) {
            _.chain(drop.getNestedDrops())
                .sortBy(function (nestedDrop) {
                    return nestedDrop.data.naam;
                })
                .each(function (nestedDrop) {
                    $('<div />')
                        .addClass('propertiesEditorsSubTitel')
                        .text(nestedDrop.data.naam)
                        .appendTo($drop);

                    var $section = $('<div />')
                        .addClass('propertiesEditors')
                        .appendTo($drop);

                    self.callDropPropertyEditors(nestedDrop, function (e) {
                        e.prepare(self);
                        e.render($drop);
                    });
                })
                .value();
        }

        $drop.append($('<div />').addClass('seperator'));
    };

    DropContainerEditor.prototype.callDropPropertyEditors = function (drop, callback) {
        _.chain(drop.propertyEditors)
            .filter(function (propertyEditor) {
                return propertyEditor.propertyCanBeEdited();
            })
            .each(callback)
            .value();
    };

    DropContainerEditor.prototype.getNestedDrops = function () {
        var drops = [];

        _.each(this.dropList.pool, function(drop) {
            drops.push(drop);
            if (drop.getNestedDrops)
                [].push.apply(drops, drop.getNestedDrops());
        });

        return drops;
    };

    return DropContainerEditor;
});