﻿define([
    'jquery',
    'nico/drop',
    './dropDown',
    'nico/Designer/propertyEditors'
], function ($, Drop, DropDown, propertyEditors) {
    'use strict';

    function DropDownDesign(options, $element) {
        DropDown.call(this, options, $element);

        if (this.data.tekstInherited)
            this.$element
                .addClass('inheritedTekst');

        this.initAnchorData(this.data);
        this.addClicksForEdit(this.getPropertyEditors());
    }

    Drop.extend(DropDownDesign, DropDown);

    DropDownDesign.prototype.addSaveData = function (data) {
        data.hasVariabele = true;
        data.variabeleNaam = this.data.variabele.naam;
        data.variabeleType = this.data.variabele.type;
        data.variabeleDefaultWaarde = this.data.variabele.defaultWaarde;
        data.tags = this.data.tags;
        data.keuzes = this.data.keuzes;
    };

    DropDownDesign.prototype.afterSave = function (result) {
        this.$element
            .removeClass('inheritedTekst');
        this.data.keuzes.length = 0;
        this.data.keuzes.push.apply(this.data.keuzes, result.keuzes);
    };

    DropDownDesign.prototype.createOperationRoute = "addKeuzelijstDrop";
    DropDownDesign.prototype.updateOperationRoute = "updateKeuzelijstDrop";

    DropDownDesign.prototype.getPropertyEditors = function () {
        var self = this;
        var tagProps =
            propertyEditors
                .createTags('Tags')
                .withGet(function () {
                    return self.data.tags;
                })
                .withSet(function (value) {
                    self.data.tags = value;
                })
                .withDataDecorator('tags');
        return [
            propertyEditors
                .createVariable('Variabele')
                .withGet(function () {
                    return self.data.variabele;
                })
                .withSet(function (value) {
                    self.data.variabele = value;
                })
                .OnTagChange(function (tags) { tagProps.tagsEditor.setTags(tags); self.data.tags = tags; })
                .withDataDecorator('variabele'),

            tagProps,

            propertyEditors
                .createKeyValueList(
                    'Keuze mogelijkheden',
                    function () {
                        return { label: 'label', waarde: 'waarde' };
                    },
                    function () {
                        self.renderOptions();
                    })
                .withKeyProperty('label')
                .withValueProperty('waarde')
                .withGet(function () {
                    return self.data.keuzes;
                })
                .withSet(function (value) {
                    self.data.keuzes = value;
                })
                .withDataDecorator('keuzes'),

            propertyEditors
                .createText('Default Waarde')
                .withGet(function () {
                    return self.data.variabele.defaultWaarde;
                })
                .withSet(function (value) {
                    self.data.variabele.defaultWaarde = value;
                }),

            propertyEditors
                .createBool('Verplicht')
                .withGet(function () {
                    return self.data.verplicht;
                })
                .withSet(function (value) {
                    self.data.verplicht = value;
                })
                .withDataDecorator('verplicht')
        ];
    };

    return DropDownDesign;
});