﻿define([
    'jquery',
    'underscore',
    'nico/Designer/PropertyEditors/propertyEditor',
    'nico/Designer/fileUploader',
    'nico/url'
], function ($, _, PropertyEditor, FileUploader, url) {
    'use strict';

    function KeuzelijstMetAfbeeldingenPropertyEditor(keuzelijst, label) {
        PropertyEditor.call(this, label);
        this.keuzelijst = keuzelijst;
    }

    PropertyEditor.extend(KeuzelijstMetAfbeeldingenPropertyEditor, PropertyEditor);

    KeuzelijstMetAfbeeldingenPropertyEditor.propertyEditorName = 'KeuzelijstMetAfbeeldingenPropertyEditor';

    KeuzelijstMetAfbeeldingenPropertyEditor.prototype.render = function ($body) {
        if (this.$table)
            this.$table.remove();

        this.list = this.getValue();
        this.$table = $('<div />')
            .addClass('table')
            .css('margin', '0 auto')
            .append($('<div />')
                .addClass('bold')
                .addClass('tableRow')
                .append($('<div />').addClass('tableCell'))
                .append($('<div />').addClass('tableCell').text('Tekst'))
                .append($('<div />').addClass('tableCell').text('Waarde'))
                .append($('<div />').addClass('tableCell').text('Afbeelding'))
                .append($('<div />').addClass('tableCell')));

        var self = this;
        var $rows = _.map(this.list, function (keyValue) {
            return self.createRow(keyValue);
        });

        this.$table
            .append($rows)
            .appendTo($body);
    };

    KeuzelijstMetAfbeeldingenPropertyEditor.prototype.createRow = function (keuzeOptie) {
        var self = this;
        var $row = $('<div />');

        var $label = $('<input type="text" />')
            .val(keuzeOptie.label)
            .addClass('short')
            .keyup(function () {
                keuzeOptie.label = $label.val();
                self.keuzelijst.renderOptions();
            })
            .change(function () {
                keuzeOptie.label = $label.val();
                self.keuzelijst.renderOptions();
            });

        var $waarde = $('<input type="text" />')
            .val(keuzeOptie.waarde)
            .addClass('short')
            .keydown(function (e) {
                if (self.valueCanHaveWhiteSpace)
                    return true;
                return e.which !== 32;
            })
            .keyup(function () {
                keuzeOptie.waarde = $waarde.val();
                self.keuzelijst.renderOptions();
            })
            .change(function () {
                if (!self.valueCanHaveWhiteSpace) {
                    $(this).val(function (i, v) {
                        return v.replace(/ /g, '');
                    });
                }
                keuzeOptie.waarde = $waarde.val();
                self.keuzelijst.renderOptions();
            });

        var $afbeelding = $('<div />')
            .append($('<div/>').text(keuzeOptie.afbeelding ? 'Een andere afbeelding kiezen' : 'Een afbeelding kiezen'));

        new FileUploader($afbeelding, function (filename) {
            keuzeOptie.afbeelding = filename;
            keuzeOptie.url = url.toTempUrl(filename);
            self.keuzelijst.renderOptions();
        });

        var $deleteEntry = $('<button />')
            .addClass('designerButton small delete')
            .click(function () {
                if (self.list.length === 1)
                    return;

                var rowIndex = self.list.indexOf(keuzeOptie);
                self.list.splice(rowIndex, 1);
                $row.remove();
                self.keuzelijst.renderOptions();
            });

        var $newEntry = $('<button />')
           .addClass('designerButton small add')
           .css('vertical-align', 'center')
           .click(function () {
               var rowIndex = self.list.indexOf(keuzeOptie);
               var newElement = self.createNewElement();
               self.list.splice(rowIndex + 1, 0, newElement);

               var $new = self.createRow(newElement);
               $new.insertAfter($row);
               self.updateView();
           });

        return $row
            .addClass('tableRow')
            .append($('<div />')
                .addClass('tableCell verticalAlign')
                .css('width', '26px')
                .append($deleteEntry))
            .append($('<div />')
                .addClass('tableCell')
                .append($label))
            .append($('<div />')
                .addClass('tableCell')
                .append($waarde))
            .append($('<div />')
                .addClass('tableCell')
                .append($afbeelding))
            .append($('<div />')
                .addClass('tableCell verticalAlign')
                .append($newEntry));
    };

    KeuzelijstMetAfbeeldingenPropertyEditor.prototype.createNewElement = function() {
        return {
            label: 'label',
            waarde: 'waarde',
            afbeelding: null
        };
    };

    return KeuzelijstMetAfbeeldingenPropertyEditor;
});