﻿define([
    'jquery',
    'nico/drop',
    './../VraagStelling/vraagStelling'
], function ($, Drop, VraagStelling) {
    'use strict';

    var defaults = function() {
        return {
            fixedVariabeleType: true,
            variabele: {
                type: 'number'
            },
            stapWaarde: 1,
            startWaarde: null,
            roundToNearestStap: false,
            rangeStart: null,
            rangeEnd: null,
            verplicht: false
        };
    };

    function NumeriekeInput(options, $element) {
        VraagStelling.call(this, $.extend(defaults(), options), $element);
        
        this.stapperDisabled = false;
        this.buttonHandlersOn = true;
    };

    Drop.extend(NumeriekeInput, VraagStelling);
    
    NumeriekeInput.prototype.clearValue = function () {
        this.$input.val(null);
    };

    NumeriekeInput.prototype.setEenheid = function (eenheid) {
        this.$eenheid.text(eenheid);
    };

    NumeriekeInput.prototype.updateValue = function (factor) {
        if (!this.buttonHandlersOn)
            return;

        var newVal = this.getNextValue(factor);

        if (this.data.rangeStart != null && newVal < this.data.rangeStart)
            newVal = this.data.rangeStart;
        else if (this.data.rangeEnd != null && newVal > this.data.rangeEnd)
            newVal = this.data.rangeEnd;

        this.$input.val(newVal);
        this.registreerAntwoord(newVal);
    };

    NumeriekeInput.prototype.getNextValue = function (factor) {
        if (this.$input.numericVal() === null) {
            if (this.data.startWaarde !== null)
                return this.data.startWaarde;
            if (this.data.rangeStart !== null)
                return this.data.rangeStart;
            return 0;
        }

        var currentValue = this.$input.numericVal();

        if (this.data.rangeStart != null && currentValue < this.data.rangeStart)
            return this.data.rangeStart;

        if (this.data.rangeEnd != null && currentValue > this.data.rangeEnd)
            return this.data.rangeEnd;

        if (this.data.roundToNearestStap) {
            var remainder = this.getRemainder(currentValue);
            if (remainder !== 0)
                return currentValue + (factor * Math.abs(remainder));
        } 
        var val = this.$input.numericVal().valueOf();
        var change = (factor * this.data.stapWaarde);
        return (val * 10  + change * 10) / 10;
    };

    NumeriekeInput.prototype.getRemainder = function (value) {
        var min = this.data.rangeStart !== null ? this.data.rangeStart : 0;
        return (value - min) % this.data.stapWaarde;
    };

    NumeriekeInput.prototype.getMinValue = function() {
        return this.data.rangeStart !== null ? this.data.rangeStart : 0;
    };

    NumeriekeInput.prototype.preRender = function () {
        this.applyPosition();
    };

    NumeriekeInput.prototype.render = function () {
        var id = _.uniqueId('numeriekeInput');

        var self = this;

        this.$decrease = $('<a />')
            .addClass('icon icon-minus qt-minus')
            .on('mousedown', function () {
                self.updateValue(-1);
            });

        this.$input = $('<input type="text" id="' + id + '" />')
            .val(self.getInitieelAntwoord())
            .numericInput({
                minValue: this.data.rangeStart,
                maxValue: this.data.rangeEnd,
                valueChanged: function (val) {
                    self.registreerAntwoord(val);
                },
                valid: function () {
                    self.buttonHandlersOn = true;
                },
                invalid: function () {
                    self.buttonHandlersOn = false;
                }
            });

        this.$increase = $('<a />')
            .addClass('icon icon-plus qt-add')
            .on('mousedown', function () {
                self.updateValue(1);
            });

        this.$eenheid = $('<label />')
            .attr('for', id)
            .addClass('unit')
            .text(this.data.eenheid);

        this.$element
            .append(this.$decrease, this.$input, this.$increase, this.$eenheid);
    };

    return NumeriekeInput;
});