﻿define([
    'jquery',
    'nico/drop',
    './opslaanKnop',
    'nico/Designer/propertyEditors',
    'nico/Designer/buttonDesign'
], function ($, Drop, OpslaanKnop, propertyEditors, ButtonDesign) {
    'use strict';

    function OpslaanKnopDesign(options, $element) {
        OpslaanKnop.call(this, options, $element);

        this.buttonDesign = new ButtonDesign(this.data);

        if (this.data.buttonType === 'tekst' && this.data.tekstInherited)
            this.$element.addClass('inheritedTekst');

        this.initAnchorData(this.data);
        this.addClicksForEdit(this.getPropertyEditors());

        var self = this;
        this.addDataDecorator(function (dropData) {
            dropData.buttonType = self.data.buttonType;
        });
    };

    Drop.extend(OpslaanKnopDesign, OpslaanKnop);

    OpslaanKnopDesign.prototype.handleClick = $.noop;

    OpslaanKnopDesign.prototype.addSaveData = function (data) {
        this.buttonDesign.addSaveData(data);
        
        data.conditie = this.data.conditie;

        if (this.$tekst)
            data.tekst = this.$tekst.html();
    };

    OpslaanKnopDesign.prototype.getPropertyEditors = function () {
        var self = this;
        var properties = [];

        if (this.data.buttonType === 'tekst') {
            properties.push(propertyEditors.createText('Knop tekst')
                .withGet(function () {
                    return self.$tekst.text();
                })
                .withSet(function (value) {
                    self.$tekst.text(value);
                })
                .withDataDecorator('tekst'));
        }

        if (this.data.buttonType === 'icon') {
            properties.push(propertyEditors.createIcon('Icon')
                .withGet(function() {
                    return self.data.icon;
                })
                .withSet(function(value) {
                    self.setIcon(value);
                    self.data.icon = value;
                })
                .withDataDecorator('icon'));
        }

        properties.push(propertyEditors.createTextArea('Conditie')
            .withGet(function () {
                return self.data.conditie;
            })
            .withSet(function (value) {
                self.data.conditie = value;
            })
            .onlyShowWhen(function () {
                return self.data.conditional;
            })
            .withDataDecorator('conditie'));

        properties.push(propertyEditors.createCheckBox('\'Hard Refresh\'')
                .withGet(function () {
                    return self.data.hardRefresh;
                })
                .withSet(function (value) {
                    self.data.hardRefresh = value;
                })
                .withDataDecorator('hardRefresh'));

        return this.buttonDesign.decorateEditors(properties);
    };

    OpslaanKnopDesign.prototype.afterSave = function () {
        this.$element.removeClass('inheritedTekst');
    };

    OpslaanKnopDesign.prototype.createOperationRoute = 'addOpslaanKnop';
    OpslaanKnopDesign.prototype.updateOperationRoute = 'updateOpslaanKnop';

    return OpslaanKnopDesign;
});