﻿define([
    'jquery',
    'nico/drop',
    './popupKnop',
    './popupDesigner',
    'nico/Designer/propertyEditors',
    'widgets/Designer/widgetListLayoutDesigner'
], function ($, Drop, PopupKnop, PopupDesigner, propertyEditors, WidgetListLayoutDesigner) {
    'use strict';

    function PopupKnopDesign(options, $element) {
        PopupKnop.call(this, options, $element);
        this.data = $.extend(this.data, options);

        if (this.data.tekstInherited)
            this.$element
                .addClass('inheritedTekst');

        this.initAnchorData(this.data);
        this.addClicksForEdit(this.getPropertyEditors());
    }

    Drop.extend(PopupKnopDesign, PopupKnop);

    PopupKnopDesign.prototype.handleClick = $.noop;

    PopupKnopDesign.prototype.openPopup = function () {
        var self = this;

        if (!this.widgetLayout) {
            this.widgetLayout = new WidgetListLayoutDesigner(this.data.drops, this.$popupContent);

            this.widgetLayout.setSaveDataDecorator(function (data) {
                self.decorateSaveData(data);
                data.path.push(self.id);
            });
            this.widgetLayout.render();
        }

        this.popup
            .setContent(this.$popupContent)
            .show({ doNotAutoClose: true, doNotCenter: true, allowScrolling: true })
            .show()
            .promise()
            .done(function () {
                self.$popupContent.detach();
            });
    };
    
    PopupKnopDesign.prototype.getPropertyEditors = function () {
        var self = this;

        return [

            propertyEditors.createText('Knop tekst')
                .withGet(function() {
                    return self.$tekst.text();
                })
                .withSet(function(value) {
                    self.$tekst.text(value);
                })
                .withDataDecorator('tekst'),

            new PopupDesigner(self, this.$popupContent)

        ];
    };

    PopupKnopDesign.prototype.addSaveData = function (data) {
        data.tekst = this.$tekst.html();
    };

    PopupKnopDesign.prototype.afterSave = function () {
        this.$element.removeClass('inheritedTekst');
    };

    PopupKnopDesign.prototype.createOperationRoute = 'addPopupKnopDrop';
    PopupKnopDesign.prototype.updateOperationRoute = 'updatePopupKnopDrop';

    return PopupKnopDesign;
});