﻿define([
    'jquery',
    'underscore',
    'nico/Designer/PropertyEditors/propertyEditor'
], function ($, _, PropertyEditor) {
    'use strict';

    function SliderStepsPropertyEditor(label) {
        PropertyEditor.call(this, label);
        this.steps = null;
        this.$steps = null;
    }

    PropertyEditor.extend(SliderStepsPropertyEditor, PropertyEditor);

    SliderStepsPropertyEditor.propertyEditorName = 'SliderStepsPropertyEditor';

    SliderStepsPropertyEditor.prototype.render = function ($body) {
        if (this.$table)
            this.$table.remove();

        this.steps = this.getValue();
        this.$steps = [];

        this.$table = $('<div />')
            .addClass('table')
            .css('margin', '0 auto')
            .append($('<div />')
                .addClass('bold')
                .addClass('tableRow')
                .append($('<div />').addClass('tableCell'))
                .append($('<div />').addClass('tableCell').text('Waarde'))
                .append($('<div />').addClass('tableCell')));

        var self = this;
        var $rows = _.map(this.steps, function (step, index) {
            return self.createRow(step, index);
        });

        this.$table
            .append($rows)
            .appendTo($body);
    };

    SliderStepsPropertyEditor.prototype.createRow = function (step, index) {
        var self = this;
        var $row = $('<div />');

        var $step = $('<input type="text" />')
            .val(step)
            .keydown(function (e) {
                return e.which !== 32;
            })
            .keyup(function () {
                var rowIndex = self.$steps.indexOf($step);
                self.steps[rowIndex] = $step.val();
            })
            .change(function () {
                $(this).val(function (i, v) {
                    return v.replace(/ /g, '');
                });
                var rowIndex = self.$steps.indexOf($step);
                self.steps[rowIndex] = $step.val();
            });
        this.$steps.splice(index + 1, 0, $step);

        var $deleteEntry = $('<button />')
            .addClass('designerButton small delete')
            .click(function () {
                if (self.$steps.length === 1)
                    return;

                var rowIndex = self.$steps.indexOf($step);
                self.$steps.splice(rowIndex, 1);
                self.steps.splice(rowIndex, 1);
                $row.remove();
            });

        var $newEntry = $('<button />')
           .addClass('designerButton small add')
           .css('vertical-align', 'center')
           .click(function () {
               var rowIndex = self.$steps.indexOf($step);
               self.steps.splice(rowIndex + 1, 0, '');

               var $new = self.createRow('', rowIndex);
               $new.insertAfter($row);
           });

        return $row
            .addClass('tableRow')
            .append($('<div />')
                .addClass('tableCell verticalAlign')
                .css('width', '26px')
                .append($deleteEntry))
            .append($('<div />')
                .addClass('tableCell')
                .append($step))
            .append($('<div />')
                .addClass('tableCell verticalAlign')
                .append($newEntry));
    };

    return SliderStepsPropertyEditor;
});