﻿define([
    'jquery',
    'underscore',
    'nico/drop'
], function ($, _, Drop) {
    'use strict';

    var defaults = function () {
        return {
            titel: '',
            tekst: '',
            icon: null
        };
    };

    function Tile(options, $element) {
        Drop.call(this, $element, defaults());

        this.data = $.extend(this.data, options);

        var self = this;
        this.$element
            .text(this.data.titel)
            .click(function (e) {
                e.preventDefault();

                if (self.isOpen())
                    self.close(true);
                else
                    self.open();
            });
    };

    Drop.extend(Tile, Drop);

    Tile.prototype.preRender = function () {
        this.applyPosition();
    };

    Tile.prototype.isOpen = function () {
        return this.$element.hasClass('active');
    };

    Tile.prototype.close = function (informRootOfLayoutChange) {
        this.$element.removeClass('active');

        var self = this;
        this.$content.slideUp('slow', function () {
            if (informRootOfLayoutChange === true)
                self.getRootDrop().raiseEvent('layoutUpdated');
        });
    };

    Tile.prototype.open = function () {
        if (!this.$content)
            this.$content = this.buildContent();
        this.closeAnyOtherTile();

        this.$element.addClass('active');

        var self = this;
        this.$content.slideDown('slow', function () {
            self.getRootDrop().raiseEvent('layoutUpdated');
        });
    };

    Tile.prototype.buildContent = function () {
        var $content = $('<div />')
            .addClass('tile-content');

        if (this.data.icon) {
            $content
                .addClass('text-icon')
                .append($('<div />').addClass('icons')
                    .append($('<span />').addClass('icon').addClass(this.data.icon)))
                .append($('<div />').addClass('content').html(this.data.tekst));
        } else {
            $content.html(this.data.tekst);
        }

        return $content
            .appendTo(this.$element.parents('.widget').find('.tiles-content'));
    };

    Tile.prototype.closeAnyOtherTile = function () {
        var self = this;
        _.each(this.parent.pool, function (drop) {
            if (drop.type === self.type && drop.isOpen())
                drop.close();
        });
    };

    return Tile;
});