﻿define([
    'jquery',
    'nico/drop',
    './tile',
    'nico/Designer/propertyEditors'
], function ($, Drop, Tile, propertyEditors) {
    'use strict';

    function TileDesign(options, $element) {
        Tile.call(this, options, $element);

        if (this.data.tekstInherited)
            this.$element
                .addClass('inheritedTekst');

        this.initAnchorData(this.data);
        this.addClicksForEdit(this.getPropertyEditors());
    };

    Drop.extend(TileDesign, Tile);

    TileDesign.prototype.getPropertyEditors = function () {
        var self = this;

        return [

            propertyEditors.createText('Titel')
                .withGet(function () {
                    return self.$element.text();
                })
                .withSet(function (value) {
                    self.$element.text(value);
                })
                .withDataDecorator('titel'),

            propertyEditors.createRichText('Tekst')
                .withGet(function () {
                    return self.data.tekst;
                })
                .withSet(function (value) {
                    self.data.tekst = value;
                })
                .withDimensions(640, 80)
                .withResizeButton(640, 560)
                .withDataDecorator('tekst'),

            propertyEditors.createIcon('Icon')
                .withGet(function () {
                    return self.data.icon;
                })
            .withSet(function (value) {
                self.data.icon = value;
            })
            .withDataDecorator('icon'),

            propertyEditors.createTextArea('Conditie')
                .withGet(function () {
                    return self.data.conditie;
                })
                .withSet(function (value) {
                    self.data.conditie = value;
                })
                .onlyShowWhen(function () {
                    return self.data.conditional;
                })
                .withDataDecorator('conditie')

        ];
    };

    TileDesign.prototype.addSaveData = function (data) {
        data.tekst = this.data.tekst;
        data.titel = this.$element.text();
        data.conditie = this.data.conditie;
    };

    TileDesign.prototype.afterSave = function () {
        this.$element
            .removeClass('inheritedTekst');
    };

    TileDesign.prototype.createOperationRoute = "addTileDrop";
    TileDesign.prototype.updateOperationRoute = "updateTileDrop";

    return TileDesign;
});