﻿define([
    'jquery',
    'nico/drop',
    'fitVids' // for video player
], function ($, Drop, fitVids) {
    'use strict';

    var defaults = function () {
        return {
            embedCode: ''
        };
    };

    function Video(options, $element) {
        Drop.call(this, $element, defaults());

        this.data = $.extend(this.data, options);

        this.renderedPromise = $.Deferred();
    };

    Drop.extend(Video, Drop);

    Video.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    Video.prototype.preRender = function () {
        this.applyPosition();
        this.prepareVideo(this.data.embedCode);
    };

    Video.prototype.prepareVideo = function (embedCode) {
        if (this.$player) {
            // cleaning up after fitVids
            this.$element
                .find('.fluid-width-video-wrapper')
                .remove();
            this.$player = null;
        }

        if (embedCode) {
            this.$player = $(embedCode);
            this.makeSureLinkHasAProtocolOtherwiseItWontWorkInPhonegap(this.$player);
            this.$player.appendTo(this.$element);

            var self = this;
            _.defer(function () {
                self.$element.fitVids();
                self.renderedPromise.resolve();
            });
        } else {
            this.renderedPromise.resolve();
        }
    };

    Video.prototype.makeSureLinkHasAProtocolOtherwiseItWontWorkInPhonegap = function ($player) {
        var url = $player.attr('src');

        // note: the default YouTube embed code does not have a protocol specified in the url
        //  -> on a device the default protocol is 'file:'
        if (!/^(?:f|ht)tps?\:/.test(url)) {
            if (window.location.protocol === 'https:')
                url = window.location.protocol + url;
            else
                url = 'http:' + url;
            $player.attr('src', url);
        }
    }

    return Video;
});