﻿define([
    'jquery',
    'nico/observable',
    'nico/Designer/propertyEditors'
], function ($, observable, propertyEditors) {
    'use strict';

    function VraagStellingDesign() {
        observable.makeObservable(this);
    };

    VraagStellingDesign.prototype.decorateSaveData = function (data, dropData) {
        data.hasVariabele = true;
        data.variabeleNaam = dropData.variabele.naam;
        data.variabeleType = dropData.variabele.type;
        data.variabeleDefaultWaarde = dropData.variabele.defaultWaarde;
        data.toonVorigeWaarde = dropData.toonVorigeWaarde;
        data.tags = dropData.variabele.tags;
    };

    VraagStellingDesign.prototype.decorateEditors = function (defaultWaardeEditor, otherEditors, data) {
        var self = this;

        var tagPropertyEditor =
            propertyEditors
                .createTags('Tags')
                .withGet(function() {
                    return data.variabele.tags;
                })
                .withSet(function(value) {
                    data.variabele.tags = value;
                })
                .withDataDecorator('tags');

        var variabelePropertyEditor =
            propertyEditors
                .createVariable('Variabele')
                .withGet(function() {
                    return data.variabele;
                })
                .withSet(function(value) {
                    data.variabele = value;
                    self.informListeners('variabeleUpdated', value);
                })
                .OnTagChange(function(tags) {
                    tagPropertyEditor.tagsEditor.setTags(tags);
                    data.variabele.tags = tags;
                })
                .withDataDecorator('variabele');

        if (data.fixedVariabeleType)
            variabelePropertyEditor.cannotEditTypeOfVariable();

        var arEditors = [
            variabelePropertyEditor,

            defaultWaardeEditor
                .withGet(function() {
                    return data.variabele.defaultWaarde;
                })
                .withSet(function(value) {
                    data.variabele.defaultWaarde = value;
                }),

            propertyEditors
                .createCheckBox('Toon Vorige Waarde')
                .withGet(function() {
                    return data.toonVorigeWaarde;
                })
                .withSet(function(value) {
                    data.toonVorigeWaarde = value;
                }),

            tagPropertyEditor
        ];

        if (!data.noNeedToShowVerplichtEditor)
            arEditors.push(
                propertyEditors
                .createBool('Verplicht')
                .withGet(function() {
                    return data.verplicht;
                })
                .withSet(function(value) {
                    data.verplicht = value;
                })
                .withDataDecorator('verplicht'));

        return  $.merge(arEditors, otherEditors);
    };

    return VraagStellingDesign;
});