﻿define([
    'jquery',
    'underscore',
    'nico/drop',
    'nico/observable',
    'nico/dropFactory',
    './templateTraverser'
], function ($, _, Drop, observable, dropFactory, TemplateTraverser) {
    'use strict';

    function Widget(data, $element) {
        Drop.call(this, $element, data);

        this.renderedPromise = $.Deferred();
    };

    Drop.extend(Widget, Drop);

    Widget.prototype.environment = 'runtime';

    Widget.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    Widget.prototype.preRender = function () {
        var drops = this.getAllDropsThatNeedToRestored();
        this.renderDrops(drops);
    };

    Widget.prototype.renderDrops = function (drops) {
        var self = this;

        if (drops.length === 0) {
            this.renderedPromise.resolve();
        } else {
            _.each(drops, function (drop) {
                drop.data.dropRuntimeData.quizMaster = self.data.quizMaster;
                drop.data.dropRuntimeData.conditional = self.data.conditional;
                dropFactory
                    .restoreDrop(self.environment, 'buildingBlocks', drop.data, drop.$element)
                    .done(function (drop) {
                        self.addDrop(drop);
                        
                        if (self.pool.length === drops.length) {
                            var promises = [];
                            _.each(self.pool, function (drop) {
                                drop.render();

                                if (_.isFunction(drop.promise))
                                    promises.push(drop.promise());
                            });

                            if (promises.length === 0) {
                                self.renderedPromise.resolve();
                            } else {
                                $.when
                                    .apply($, promises)
                                    .done(function() {
                                        self.renderedPromise.resolve();
                                    });
                            }
                        };
                    });
            });
        }
    };

    Widget.prototype.getAllDropsThatNeedToRestored = function () {
        var drops = [];

        var self = this;
        _.each(this.getDropNodes(), function ($node) {
            var data = self.getExistingDrop($node);
            if (data) {
                drops.push({
                    data: data,
                    $element: $node
                });
            } else {
                $node.addClass('hidden');
            }
        });

        return drops;
    };

    Widget.prototype.getExistingDrop = function ($node) {
        var naam = $node.data('naam');
        var existingDrop = _.find(this.data.drops, function (dropData) {
            return dropData.naam === naam;
        });
        if (!existingDrop)
            return null;

        // note: in some cases this will allow us to change the type of the drop
        // in the template without having to write a migration
        existingDrop.type = this.getExpectedTypeOfDrop($node);
        return existingDrop;
    };

    Widget.prototype.getExpectedTypeOfDrop = function ($node) {
        if (this.allowsMultipleDrops($node))
            return 'dropContainer';
        return $node.data('type');
    };

    Widget.prototype.allowsMultipleDrops = function ($node) {
        return typeof $node.attr('data-many') !== 'undefined';
    };

    Widget.prototype.getDropNodes = function () {
        var templateTraverser = new TemplateTraverser(this.$element);
        return templateTraverser.traverse();
    };

    Widget.prototype.raiseEvent = function () {
        this.parent.raiseEvent.apply(this.parent, arguments);
    };

    return Widget;
});