define([
    'jquery',
    'nico/viewPager',
    'nico/dispatcher',
    'nico/Index/feed',
    'nico/Index/advies',
    'nico/userSettings',
    'nico/localStorage',
    'nico/menuRuntime',
    'nico/systeemBoodschappen',
    'nico/url'
], function ($, ViewPager, Dispatcher, Feed, Advies, userSettings, localStorage, MenuRuntime, systeemBoodschappen, url) {
    'use strict';
    var phoneRegistered = new $.Deferred();
    var phoneRegisteredForIos = new $.Deferred();
    var batchComplete = new $.Deferred();

    if (window.plugins && window.plugins.pushNotification) {

        var pushNotification = window.plugins.pushNotification;

        window.onGoogleNotification = function (e) {
            switch (e.event) {
                case 'registered':
                    if (e.regid.length > 0) {
                        phoneRegistered.resolve(e.regid);
                    }
                    break;
            }
        };

        window.onNotificationAPN = function (e) {
            if (e.alert) {
                navigator.notification.alert(e.alert);
            }
            if (e.badge) {
                pushNotification.setApplicationIconBadgeNumber(this.successHandler, event.badge);
            }
            if (e.sound) {
                var snd = new Media(event.sound);
                snd.play();
            }
        };
        
        var errorHandler = function (error) {
            $.post(url.getFullUrl('error'), JSON.stringify({
                message: 'Unable to register device: ' + error,
                file: null,
                lineNumber: null,
                stack: null
            }), function () { });
        };

        // source: 
        // https://github.com/hollyschinsky/PhoneGapBuildPushProject2/
        // http://devgirl.org/2013/01/24/push-notifications-plugin-support-added-to-phonegap-build/
        if (device.platform == 'android' || device.platform == 'Android') {
            var successHandler = function (result) { };
            pushNotification.register(
                successHandler,
                errorHandler,
                { "senderID": "878411654862", "ecb": "window.onGoogleNotification" });
        } else {
            var tokenHandler = function (result) {
                console.log('token received for iOS: ' + result);
                phoneRegisteredForIos.resolve(result);
            };
            pushNotification.register(
                tokenHandler,
                errorHandler,
                { "badge": "true", "sound": "true", "alert": "true", "ecb": "window.onNotificationAPN" });
        }
    }


    var start = function () {
        var batch = [
            {
                route: '/Feed/Open',
                done: function (result) {
                    var $body = $('body');
                    if (!result.programma) {
                        window.location.href = 'incomplete.html';
                    } else {
                        userSettings.update({
                            programma: result.programma
                        });

                        if (!$body.hasClass(result.programma.kleur)) {
                            $('body')
                                .removeClass()
                                .addClass(result.programma.kleur);
                        }

                        $('#title').text(result.programma.titel);

                        buildUserInterface(result);
                    }
                }
            },
            {
                route: '/Menu/Runtime/Open',
                data: {
                    location: 'mobile'
                },
                done: function (result) {
                    new MenuRuntime($('#main .title'), $('.navigationDrawer'))
                        .render(result);
                }
            },
            {
                route: '/SysteemBoodschappen/Runtime/Load',
                done: function (result) {
                    systeemBoodschappen.set(result);
                    batchComplete.resolve();
                }
            }
        ];

        new Dispatcher().postBatch(batch);

        $.when(phoneRegistered, batchComplete).done(function (phoneId) {
            new Dispatcher()
                .post('/Home/RegisterPhoneForGcm', { phoneId: phoneId });
        });

        $.when(phoneRegisteredForIos, batchComplete).done(function (phoneId) {
            new Dispatcher()
                .post('/Home/RegisterPhoneForIos', { phoneId: phoneId });
        });
    };

    var buildUserInterface = function (result) {
        var viewPager = new ViewPager($('#viewPager'));
        viewPager.render();

        $('#firstTabTekst').text(result.firstTabTekst);
        $('#firstTabIcon').addClass(result.firstTabIcon);
        $('#secondTabTekst').text(result.secondTabTekst);
        $('#secondTabIcon').addClass(result.secondTabIcon);

        $('#advies-nu').text(result.adviesNuTekst);
        
        new Feed($('#mijnFeed')).render(result.feed);
        new Advies($('#panic'), result.panicIntroductie).render();  
    };

    return {
        start: start
    };

});