define([
    'jquery',
    'nico/dispatcher',
    'nico/modalWindow',
    'nico/textReplacer',
    './loginForm',
    'nico/localStorage',
    'nico/jquery.extensions.common',
    'nouislider'
], function ($, Dispatcher, modalWindow, textReplacer, LoginForm, localStorage) {
    'use strict';

    var dispatcher = new Dispatcher();

    var start = function () {
        if (navigator.connection.type === Connection.NONE) {
            window.location.href = 'offline.html';
            return;
        }
        this.$message = $('#login-msg');
        var login = new LoginForm($('#loginForm'));
        var teksten = {};
        var token;

        var self = this;
        dispatcher
               .post('/Home/ICoach/GetRegistrationInfo', { needsToken: false })
               .done(function (result) {
                   textReplacer.apply(result.teksten, result.placeHolders);
                   _.each(result.talen, function (t) {
                       $('<option />').val(t.code).text(t.runtimeNaam).appendTo($('#taal'));
                   });
               });

        var culture = localStorage.get('nicoCulture');
        if (!culture) {
            modalWindow.getModal('choose-language').show();   
        }

        $('#disclaimer-from-voorwaarden')
            .click(function () {
                modalWindow
                    .getModal('modal-disclaimer')
                    .show();
            });

        $('#taal-instellen').click(function () {
            dispatcher
                .post('setRegistrationTaal', { taalCode: $('#taal').val() })
                .done(function (result) {
                    localStorage.set('nicoCulture', $('#taal').val());
                    teksten = result.teksten;
                    textReplacer.apply(result.teksten);
                    modalWindow.getModal('choose-language').close();
                });
        });

        $('#start-registration')
            .click(function () {
                dispatcher
                .post('/Home/ICoach/GetRegistrationInfo', { needsToken: true })
                .done(function (result) {
                    token = result.token;
                    teksten = result.teksten;
                    textReplacer.apply(result.teksten);
                    modalWindow.getModal('register-profile').show();
                });

                $('#akkoord-met-voorwaarden').noUiSlider({
                    start: 0,
                    step: 1,
                    range: {
                        'min': 0,
                        'max': 1
                    }
                });

                if (!window.plugins|| (!(device && device.platform && device.platform.toLowerCase() == 'ios'))) {
                    $('.required-field_except_for_ios').addClass('required-field');
                    $('.mandatory').show();
                }
                $('#registreer-knop').click(function () {
                    var invalid = false;
                    
                    var findMessageElementFor = function ($el) {
                        return $el.parent().prev().find('.required-msg');
                    }

                    $('.required-field').each(function (i, el) {
                        if (!$(el).val()) {
                            invalid = true;
                            findMessageElementFor($(el)).html(teksten['verplicht']);
                        } else {
                            findMessageElementFor($(el)).text('');
                        }
                    });

                    var email = $.trim($('#register-email').val());
                    var emailAgain = $.trim($('#emailagain').val());
                    if (email != emailAgain) {
                        invalid = true;
                        findMessageElementFor($('#register-email')).html(teksten['ongeldig']);
                        findMessageElementFor($('#emailagain')).html(teksten['ongeldig'] + '');
                    }

                    if ($('#akkoord-met-voorwaarden').val() != 1) {
                        invalid = true;
                        findMessageElementFor($('#akkoord-met-voorwaarden')).html(teksten['verplicht']);
                    }

                    var geboorteDatumDag = $('#geboortedatum-dag').val();
                    var geboorteDatumMaand = $('#geboortedatum-maand').val();
                    var geboorteDatumJaar = $('#geboortedatum-jaar').val();
                    if (geboorteDatumDag && geboorteDatumMaand && geboorteDatumJaar) {
                        var date = new Date(geboorteDatumJaar, geboorteDatumMaand - 1, geboorteDatumDag);
                        if (!(date.getFullYear() == geboorteDatumJaar && date.getMonth() + 1 == geboorteDatumMaand && date.getDate() == geboorteDatumDag)) {
                            invalid = true;
                            findMessageElementFor($('#geboortedatum-jaar')).html(teksten['invalid-date'] + '');
                        }
                    }

                    if (invalid) {
                        $('#register-profile > .page').scrollTop(0);
                        return;
                    }

                    var data = {
                        voornaam: $('#voornaam').val(),
                        geboorteDatumDag: geboorteDatumDag,
                        geboorteDatumMaand: geboorteDatumMaand,
                        geboorteDatumJaar: geboorteDatumJaar,
                        geslacht: $('#geslacht').val(),
                        email: email,
                        wachtwoord: $('#register-wachtwoord').val(),
                        onderwijs: $('#onderwijs').val(),
                        werk: $('#werk').val(),
                        akkoordMetVoorwaarden: $('#akkoord-met-voorwaarden').val() == 1,
                        token: token
                    };

                    dispatcher
                        .post('registerAccount', data)
                        .done(function (result) {
                            modalWindow.getModal('register-profile').close();
                            login.clear();
                            login.submit(data.email, data.wachtwoord);
                        })
                        .fail(function (msg) {
                            $('#error-message').html(msg);
                        });
                });
        });
    };

    return {
        start: start
    };

});