﻿// todo this file is shared between the web and mobile site
define([
    'jquery',
    'nico/dispatcher',
    'nico/modalWindow',
    './validator'
], function ($, Dispatcher, modalWindow, validator) {

    function Identification() {
        this.modal = modalWindow.getModal('register-identification');

        this.$email = this.modal.$window
            .find('#identification-email');

        this.$wachtwoord = this.modal.$window
            .find('#identification-password');

        this.$msg = this.modal.$window
            .find('#identification-msg');

        var self = this;
        this.$submit = this.modal.$window
            .find('#identification-submit')
            .click(function () {
                self.submit();
            });

        this.$cancel = this.modal.$window
            .find('.cancel-registration')
            .click(function () {
                self.cancel();
            });
    };

    Identification.prototype.dispatcher = new Dispatcher();

    Identification.prototype.show = function () {

        this.modal.show();

        this.$msg.hide();

        this.$email
            .val('')
            .focus();

        this.$wachtwoord
            .val('');

        this.deferred = $.Deferred();
        return this.deferred.promise();
    };

    Identification.prototype.cancel = function () {
        this.modal.close();
    };

    Identification.prototype.submit = function () {
        this.$msg.hide();

        if (!this.$email.val() || !this.$wachtwoord.val()) {
            this.$msg
                .text('E-mailadres en wachtwoord zijn verplicht')
                .show();
        } else if (this.invalidEmail()) {
            this.$msg
                .text('Het e-mailadres is niet geldig')
                .show();
        } else if (this.invalidPassword()) {
            this.$msg
                .text('Het wachtwoord is niet geldig')
                .show();
        } else {
            var email = this.$email.val();
            var wachtwoord = this.$wachtwoord.val();

            var self = this;
            this.dispatcher
                .post('/Home/ValidateEmailAddress', {
                    address: email,
                    wachtwoord: wachtwoord
                })
                .done(function (result) {
                    self.modal.close();

                    var data = {
                        email: email,
                        wachtwoord: wachtwoord
                    };

                    if (result.needsActivation) {
                        data.needsActivation = true;
                        data.gebruikerId = result.gebruikerId;
                    }

                    self.deferred.resolve(data);
                })
                .fail(function(message) {
                    self.$msg
                        .text(message)
                        .show();
                });
        }
    };

    Identification.prototype.invalidEmail = function () {
        var email = this.$email.val();
        return !validator.isValidEmail(email);
    };

    Identification.prototype.invalidPassword = function () {
        var wachtwoord = this.$wachtwoord.val();
        return !validator.isValidPassword(wachtwoord);
    };

    return Identification;
});