define([
    'jquery',
    'nico/tokenMaster',
    'nico/dispatcher',
    'nico/url'
], function ($, tokenMaster, Dispatcher, url) {
    'use strict';

    var LoginForm = function ($element, company) {
        this.company = company;
        this.$email = $element.find('#email');
        this.$wachtwoord = $element.find('#wachtwoord');
        this.$aanmelden = $element.find('#aanmelden');
        this.$message = $element.find('#login-msg');

        var self = this;
        this.$wachtwoord
            .keyup(function() {
                self.enableOrDisableAanmeldButton();
            });
        this.$email
            .keyup(function() {
                self.enableOrDisableAanmeldButton();
            });
        this.$aanmelden
            .click(function () {
                self.submit(self.$email.val(), self.$wachtwoord.val());
            });
    };

    LoginForm.prototype.dispatcher = new Dispatcher();

    LoginForm.prototype.clear = function() {
        this.$email.val('');
        this.$wachtwoord.val('');
        this.$message.text('');
    };

    LoginForm.prototype.submit = function (email, wachtwoord) {
        this.$message.text('');

        var self = this;
        this.dispatcher
            .post('/Home/RequestToken', {
                email: email,
                wachtwoord: wachtwoord
            })
            .done(function (result) {
                if (result.requiresActivation) {
                    self.company
                        .show({
                            gebruikerId: result.gebruikerId
                        })
                        .done(function() {
                            self.submit(email, wachtwoord);
                        });
                } else {
                    tokenMaster.setAuthentication(email, wachtwoord, result.token);
                    window.location.href = 'index.html';
                }
            })
            .fail(function(message) {
                self.$message.text(message);
            });
    };

    LoginForm.prototype.enableOrDisableAanmeldButton = function() {
        var email = this.$email.val();
        var wachtwoord = this.$wachtwoord.val();

        if (!email || !wachtwoord)
            this.$aanmelden
                .attr('disabled', 'disabled');
        else
            this.$aanmelden
                .removeAttr('disabled', 'disabled');
    };

    return LoginForm;
});