﻿// todo this file is shared between the web and mobile site
define([
    'jquery',
    'nico/dispatcher',
    'nico/modalWindow'
], function ($, Dispatcher, modalWindow) {

    function Profile() {
        this.modal = modalWindow.getModal('register-profile');

        this.$voornaam = this.modal.$window
            .find('#profile-voornaam');

        this.$geboortejaar = this.modal.$window
            .find('#profile-geboortejaar')
            .numericInput({
                maxValue: new Date().getFullYear()
            });

        this.$geslacht = this.modal.$window
            .find('input[name=profile-geslacht]:radio');

        this.$taal = this.modal.$window
            .find('#profile-taal');

        this.$akkoordMetVoorwaarden = this.modal.$window
            .find('#profile-akkoord');

        this.$msg = this.modal.$window
            .find('#profile-msg');

        var self = this;
        this.$submit = this.modal.$window
            .find('#profile-account-aanmaken')
            .click(function () {
                self.submit();
            });

        this.$cancel = this.modal.$window
            .find('.cancel-registration')
            .click(function () {
                self.cancel();
            });
    }

    Profile.prototype.dispatcher = new Dispatcher();

    Profile.prototype.show = function (data) {
        this.data = data;

        this.modal.show();

        this.$msg.hide();

        this.$voornaam
            .val('')
            .focus();

        this.$geboortejaar
            .val('');
        this.$taal
            .val('nl');
        this.$akkoordMetVoorwaarden
            .val(false);
        this.$geslacht
            .removeAttr('checked');

        this.deferred = $.Deferred();
        return this.deferred.promise();
    };

    Profile.prototype.cancel = function () {
        this.modal.close();
    };

    Profile.prototype.submit = function () {
        this.$msg.hide();

        if (this.inputIsNotValid()) {
            this.$msg
                .text('Alle velden moeten ingevuld worden')
                .show();
        } else if (this.doesNotAcceptTheTerms()) {
            this.$msg
                .text('Je moet akkoord gaan met de algemene voorwaarden en de privacy voorwaarden')
                .show();
        } else {
            var data = this.data;
            data.voornaam = this.$voornaam.val();
            data.geboortejaar = this.$geboortejaar.val();
            data.geslacht = this.$geslacht.filter(':checked').val();
            data.taal = this.$taal.val();
            data.akkoordMetVoorwaarden = this.$akkoordMetVoorwaarden.prop('checked');

            var self = this;
            this.dispatcher
                .post('/Home/CreateAccount', data)
                .done(function (result) {
                    data.gebruikerId = result;

                    self.modal.close();
                    self.deferred.resolve(data);
                })
                .fail(function (message) {
                    self.$msg
                        .text(message)
                        .show();
                });
        }
    };

    Profile.prototype.inputIsNotValid = function () {
        if (this.$voornaam.val().length < 1)
            return true;

        if (this.$geboortejaar.val().length < 1)
            return true;

        if (this.$geslacht.filter(':checked').length !== 1)
            return true;

        if (this.$taal.val().length < 1)
            return true;

        return false;
    };

    Profile.prototype.doesNotAcceptTheTerms = function () {
        return !this.$akkoordMetVoorwaarden.prop('checked');
    };

    return Profile;
});