define([
    'jquery',
    'underscore',
    'nico/observable'
],function($, _, observable) {
    'use strict';

    var MeerkeuzeVraag = function(data) {
        observable.makeObservable(this);
        this.answer = null;
        this.data = data;
        this.$element = $('<div />')
            .addClass('meerKeuzeVraag');
    };

    MeerkeuzeVraag.prototype.render = function($target) {
        $('<div />')
            .addClass('vraagstelling')
            .text(this.data.vraag)
            .appendTo(this.$element);

        if(this.data.visueleStijl === 'buttons')
            this.renderListOfButtons(this.data.keuzes);
        else
            this.renderListOfRadioButtons(this.data.keuzes);

        this.$element
            .appendTo($target);

        this.informListeners('rendered');
    };

    MeerkeuzeVraag.prototype.renderListOfButtons = function(keuzes)  {
        var self = this;

        var $keuzes = _.map(keuzes, function(keuze) {
            return $('<div />')
                .addClass('button green big clickable optie')
                .text(keuze.label)
                .click(function () {
                    self.setAnswer(keuze.waarde);
                    self.submitAnswer();
                });
        });

        this.$element.append($keuzes);
    };

    MeerkeuzeVraag.prototype.renderListOfRadioButtons = function(keuzes) {
        var self = this;

        var $keuzes = _.map(keuzes, function (keuze) {
            var id = _.uniqueId('radioButton');

            var $radioButton = $('<input type="radio" name="antwoord" id="' + id + '" />')
                .val(keuze.waarde)
                .change(function () {
                    self.setAnswer(keuze.waarde);
                });

            var $label = $('<label />')
                .attr('for', id)
                .addClass('label')
                .text(keuze.label);

            return $('<li />')
                .addClass('optie clickable')
                .click(function () {
                    if(!$radioButton.prop('checked')) {
                        $radioButton.prop('checked',true);
                        self.setAnswer(keuze.waarde);
                    }
                })
                .append($radioButton, $label);
        });

        var $okButton = $('<div />')
            .addClass('button green big clickable optie')
            .text('Ok')
            .click(function() {
                self.submitAnswer();
            });

        this.$element.append($('<div />')
            .append($('<ul />').addClass('optionList').append($keuzes))
            .append($okButton));
    };

    MeerkeuzeVraag.prototype.setAnswer = function(answer) {
        this.answer = answer;
    };

    MeerkeuzeVraag.prototype.submitAnswer = function() {
        if(this.answer === null)
            return;
        this.informListeners('answered', this.answer);
    };

    MeerkeuzeVraag.prototype.destroy = function () {
        this.$element.remove();
    };

    return MeerkeuzeVraag;
});