define([
    'jquery',
    'nico/observable',
    'nico/dispatcher',
    'widgets/Runtime/widgetListLayoutRuntime',
    'nico/quizMaster'
], function ($, observable, Dispatcher, WidgetListLayoutRuntime, QuizMaster) {
    'use strict';

    var Resultaat = function(data) {
        observable.makeObservable(this);
        this.data = data;
        this.contentName = data.contentName;
        this.currentCard = null;
        this.$element = $('<div />')
            .addClass('panicOptions');
    };

    Resultaat.prototype.dispatcher = new Dispatcher();

    Resultaat.prototype.render = function($target) {
        this.$container = $('<div />')
            .addClass('block panicAdvice');

        var self = this;
        this.$ok = $('<div />')
            .addClass('button green big clickable okButton')
            .text(self.data.okTekst)
            .click(function () {
                self.userHasFoundSomePeaceOfMind();
            });

        this.$element
            .append(this.$container, this.$ok)
            .appendTo($target);

        if (this.data.secondScreen)
            this.$ok.hide();

        this.showContent(this.data.drops);
    };

    Resultaat.prototype.showContent = function (drops) {
        var self = this;

        this.quizMaster = new QuizMaster();
        this.currentCard = new WidgetListLayoutRuntime(drops, this.$container, this.quizMaster);

        this.currentCard.addListener('screenSwitched', function () {
            self.switchScreen();
        }, this);

        this.currentCard.addListener('antwoordenOpgeslagen', function () {
            self.userHasFoundSomePeaceOfMind();
        }, this);

        this.currentCard.render();
    };

    Resultaat.prototype.switchScreen = function () {
        var self = this;
        this.dispatcher
            .post('getCardContent', {
                cardId: this.data.cardId,
                contentName: this.switchContentName()
            })
            .done(function (result) {
                self.contentName = result.contentName;
                self.removeCurrentCard();
                self.$ok.show();
                self.showContent(result.drops);
            });
    };

    Resultaat.prototype.switchContentName = function () {
        return this.contentName === 'back' ? 'back2' : 'back';
    };

    Resultaat.prototype.userHasFoundSomePeaceOfMind = function () {
        if (this.quizMaster.hasAntwoorden()) {
            var self = this;
            this.dispatcher
                .post('antwoordenOpslaan', {
                    antwoorden: this.quizMaster.getAntwoorden()
                })
                .done(function () {
                    self.backToFeed();
                });
        } else {
            this.backToFeed();
        }
    };

    Resultaat.prototype.backToFeed = function () {
        window.location.href = 'index.html';
    };

    Resultaat.prototype.removeCurrentCard = function () {
        this.currentCard.removeListener('screenSwitched', this);
        this.currentCard.removeListener('antwoordenOpgeslagen', this);
        this.currentCard.close();
    };

    Resultaat.prototype.destroy = function () {
        this.removeCurrentCard();
        this.$element.remove();
    };

    return Resultaat;
});