﻿// todo this file is shared between the web and mobile site
define([
    'nico/modalWindow',
    'nico/dispatcher',
    'widgets/Runtime/widgetListLayoutRuntime',
    'nico/quizMaster'
], function (modalWindow, Dispatcher, WidgetListLayoutRuntime, QuizMaster) {
    'use strict';

    var CardBack = function (cardId, icon) {
        this.cardId = cardId;
        this.icon = icon;
        this.window = modalWindow.getModal('cardBack');
        this.$element = $('<div />');
        this.hadAntwoorden = false;
        this.widgets = null;
    };

    CardBack.prototype.dispatcher = new Dispatcher();

    CardBack.prototype.show = function () {
        var self = this;
        this.dispatcher.post('getInitialBackContent', {
            cardId: this.cardId
        })
        .done(function (content) {
            self.loadContent(content);
            self.widgets
                .promise()
                .done(function () {
                    self.window
                        .setContent(self.$element)
                        .show()
                        .promise()
                        .done(function () {
                            self.$element.detach();
                            self.promise.resolve(self.hadAntwoorden);
                            self.cleanupContent();
                            self.hadAntwoorden = false;
                        });
                });
        });

        this.promise = $.Deferred();
        return this.promise.promise();
    };

    CardBack.prototype.switchScreen = function () {
        var contentName = this.contentName === 'back' ? 'back2' : 'back';

        var self = this;
        this.dispatcher.post('getCardContent', {
            cardId: this.cardId,
            contentName: contentName
        })
        .done(function (content) {
            self.cleanupContent();
            self.loadContent(content);
        });
    };

    CardBack.prototype.loadContent = function (content) {
        this.contentName = content.contentName;
        this.widgets = new WidgetListLayoutRuntime(content.drops, this.$element, new QuizMaster());
        this.widgets.icon = this.icon;

        var self = this;
        this.widgets.addListener('screenSwitched', function (e) {
            self.hadAntwoorden = self.hadAntwoorden || e.hadAntwoorden;
            self.switchScreen();
        }, this);

        this.widgets.addListener('antwoordenOpgeslagen', function (e) {
            self.hadAntwoorden = true;

            if (!e.sourceWasAnotherWidgetList) {
                self.window.close();
            } else {
                self.lockHeight();
                self.cleanupContent();
                self.dispatcher.post('getCardContent', {
                    cardId: self.cardId,
                    contentName: self.contentName
                }).done(function (result) { self.loadContent(result); });
            }

        }, this);

        this.widgets.render();
    };

    CardBack.prototype.lockHeight = function () {
        // by keeping the height locked for a while we prevent the list from 'flickering'
        this.$element.height(this.$element.innerHeight());
    };

    CardBack.prototype.cleanupContent = function () {
        if (!this.widgets)
            return;

        this.widgets.removeListener('screenSwitched', this);
        this.widgets.removeListener('antwoordenOpgeslagen', this);
        this.widgets.close();
    };

    return CardBack;
});