﻿define([
    'nico/dispatcher',
    'widgets/Runtime/widgetListLayoutRuntime',
    'nico/quizMaster',
    './cardBack'
], function (Dispatcher, WidgetListLayoutRuntime, QuizMaster, CardBack) {

    var CardFront = function (feed, cardId, icon) {
        this.feed = feed;
        this.$element = $('<div />')
            .addClass('block');
        this.$card = $('<div />')
            .appendTo(this.$element);
        this.cardId = cardId;
        this.icon = icon;
        this.widgets = null;
    };

    CardFront.prototype.dispatcher = new Dispatcher();

    CardFront.prototype.replaceContent = function(content) {
        this.lockHeight();
        this.closeCard();
        this.showContent(content);
    };

    CardFront.prototype.showContent = function (content) {
        this.contentName = content.contentName;
        this.widgets = new WidgetListLayoutRuntime(content.drops, this.$card, new QuizMaster());
        this.widgets.icon = this.icon;

        var self = this;
        this.widgets.addListener('screenSwitched', function (e) {
            self.contentName = self.contentName === 'front' ? 'front2' : 'front';

            if (e.hadAntwoorden)
                self.reloadCardsInFeed();
            else
                self.switchScreen();
        }, this);

        this.widgets.addListener('backOfCardOpened', function (e) {
            self.showBackOfCard(e.hadAntwoorden);
        }, this);

        this.widgets.addListener('antwoordenOpgeslagen', function () {
            self.reloadCardsInFeed();
        }, this);

        this.widgets.render();
        this.unlockHeight();
    };

    CardFront.prototype.lockHeight = function() {
        // by keeping the height locked for a while we prevent the list from 'flickering'
        this.$element.height(this.$element.innerHeight());
    };

    CardFront.prototype.unlockHeight = function () {
        var self = this;
        _.defer(function () {
            self.$element.height('auto');
        });
    };

    CardFront.prototype.switchScreen = function () {
        var self = this;
        this.dispatcher.post('getCardContent', {
            cardId: this.cardId,
            contentName: this.contentName
        })
        .done(function (content) {
            self.replaceContent(content);
        });
    };

    CardFront.prototype.showBackOfCard = function (frontHadAnswers) {
        if (!this.cardBack)
            this.cardBack = new CardBack(this.cardId, this.icon);

        var self = this;
        this.cardBack
            .show()
            .done(function(backHadAntwoorden) {
                if (frontHadAnswers || backHadAntwoorden)
                    self.reloadCardsInFeed();
            });
    };

    CardFront.prototype.reloadCardsInFeed = function() {
        this.feed.reloadCards();
    };

    CardFront.prototype.closeCard = function () {
        this.$card.empty();
        this.widgets.removeListener('screenSwitched', this);
        this.widgets.removeListener('backOfCardOpened', this);
        this.widgets.removeListener('antwoordenOpgeslagen', this);
        this.widgets.close();
    };

    CardFront.prototype.destroy = function () {
        this.$element.detach();
        this.closeCard();
    };

    return CardFront;
});