﻿// todo this file is shared between the web and mobile site
define([
    'jquery',
    'nico/dispatcher',
    'widgets/Runtime/widgetListLayoutRuntime',
    'nico/quizMaster',
    'nico/modalWindow'
], function ($, Dispatcher, WidgetListLayoutRuntime, QuizMaster, modalWindow) {
    'use strict';

    var routes = {
        initialBack: 'getInitialBackContent',
        initialFront: 'getInitialFrontContent',
        specificContent: 'getCardContent'
    };

    var switches = {
        'back': 'back2',
        'back2': 'back',
        'front': 'front2',
        'front2': 'front'
    };

    var $body = $('body');

    var template = _.template($('#card-runtime-template').html());

    var CardRuntime = function (parentElement, cardId, initialKant) {
        this.parentElement = parentElement;
        this.cardId = cardId;
        this.initialKant = initialKant;
        this.hadAntwoorden = false;
        this.currentCard = null;

        this.window = modalWindow.create($(template()).appendTo($body));
        this.$element = $('<div />');
    };

    CardRuntime.prototype.dispatcher = new Dispatcher();

    CardRuntime.prototype.openCard = function () {
        var initialRoute = this.initialKant === 'achterkant' ? routes.initialBack : routes.initialFront;

        var self = this;
        this.loadInitialScreen(initialRoute)
            .done(function () {
                self.currentCard.promise().done(function () {
                    self.window
                        .setContent(self.$element)
                        .show()
                        .promise()
                        .done(function () {
                            self.$element.detach();

                            if (self.hadAntwoorden)
                                self.informParentWidgetListOfAntwoorden();

                            self.hadAntwoorden = false;

                            self.cleanupContent();
                        });
                });
            });
    };

    CardRuntime.prototype.informParentWidgetListOfAntwoorden = function () {
        if (this.parentElement.parent)
            this.parentElement.parent.raiseEvent('antwoordenOpgeslagen', { sourceWasAnotherWidgetList: true });
    };

    CardRuntime.prototype.loadInitialScreen = function (route) {
        this.cleanupContent();

        var self = this;
        return this.dispatcher.post(route, {
            cardId: this.cardId
        })
        .done(function (result) {
            self.contentName = result.contentName;
            self.renderDrops(result.drops);
        });
    };

    CardRuntime.prototype.loadSpecificScreen = function (contentName) {
        this.cleanupContent();

        var self = this;
        this.dispatcher
            .post(routes.specificContent, {
                cardId: this.cardId,
                contentName: contentName
            })
            .done(function (result) {
                self.contentName = result.contentName;
                self.renderDrops(result.drops);
            });
    };

    CardRuntime.prototype.renderDrops = function (drops) {
        this.currentCard = new WidgetListLayoutRuntime(drops, this.$element, new QuizMaster());

        var self = this;
        this.currentCard.addListener('screenSwitched', function (e) {
            self.hadAntwoorden = self.hadAntwoorden || e.hadAntwoorden;

            var contentName = switches[self.contentName];
            self.loadSpecificScreen(contentName);
        }, this);

        this.currentCard.addListener('backOfCardOpened', function (e) {
            self.hadAntwoorden = self.hadAntwoorden || e.hadAntwoorden;

            self.loadInitialScreen(routes.initialBack);
        }, this);

        this.currentCard.addListener('antwoordenOpgeslagen', function () {
            self.hadAntwoorden = true;
            self.window.close();
        }, this);

        this.currentCard.render();
    };

    CardRuntime.prototype.cleanupContent = function () {
        if (!this.currentCard)
            return;

        this.currentCard.removeListener('screenSwitched', this);
        this.currentCard.removeListener('backOfCardOpened', this);
        this.currentCard.removeListener('antwoordenOpgeslagen', this);
        this.currentCard.close();
    };

    return CardRuntime;
});