define([
    'nico/tokenMaster',
    'nico/url',
    'nico/localStorage',
    'underscore',
    'nico/busyIndicator'
], function (tokenMaster, url, localStorage, _, busyIndicator) {
    'use strict';

    var routes = [];

    var Dispatcher = function() {
    };

    Dispatcher.prototype.post = function (route, data) {
        var deferred = new $.Deferred();

        var batch = [
            {
                route: route,
                data: data,
                done: function (result) {
                    deferred.resolve(result);
                }
            }
        ];

        this.postBatch(batch)
            .fail(function (message) {
                deferred.reject(message);
            });

        return deferred.promise();
    };

    Dispatcher.prototype.postBatch = function (batch, optionalPromise) {
        var deferred = optionalPromise || new $.Deferred();

        var self = this;
        var requests = _.map(batch, function (entry) {
            return {
                route: self.getActualRoute(entry.route),
                data: entry.data || {}
            };
        });

        busyIndicator.show();

        $.ajax({
            xhrFields: { withCredentials: true },
            url: url.getFullUrl('json'),
            type: 'POST',
            cache: false,
            data: JSON.stringify(requests),
            dataType: 'json',
            beforeSend: function (request) {
                var nicoCulture = localStorage.get('nicoCulture');
                if (nicoCulture)
                    request.setRequestHeader('nicoCulture', nicoCulture);
                if (!tokenMaster.hasToken())
                    return;
                var token = tokenMaster.getToken();
                request.setRequestHeader('Authorization', 'Bearer ' + token);
            },
            contentType: 'application/json; charset=utf-8'
        })
        .complete(function () {
            busyIndicator.hide();
        })
        .done(function (results) {
            var firstFailedRequest = _.findWhere(results, { wasSuccessful: false });
            if (firstFailedRequest) {
                _.each(self.fails, function (f) {
                    f(firstFailedRequest.message);
                });

                deferred.reject(firstFailedRequest.message);
            } else {
                var receivedRoutes = _.map(results, function (result) {
                    return _.pairs(result.routes);
                });
                var allRoutes = [];
                allRoutes = allRoutes.concat.apply(allRoutes, receivedRoutes);
                if (allRoutes.length > 0) {
                    routes = {};
                    _.each(allRoutes, function (route) {
                        routes[route[0]] = route[1];
                    });
                }

                _.each(self.dones, function (f) {
                    f(results);
                });

                deferred.resolve(results);

                _.each(batch, function (entry, index) {
                    if (!entry.done)
                        return;

                    var result = results[index];
                    entry.done(result.data);
                });
            }
        })
        .fail(function (httpObj) {
            if (httpObj.status === 401) {
                tokenMaster
                    .renew()
                    .done(function () {
                        self.postBatch(batch, deferred);
                    })
                    .fail(function () {
                        window.location.href = 'login.html';
                    });
            } else {
                deferred.reject('Request failed');
            }
        });

        return deferred.promise();
    };

    Dispatcher.prototype.getActualRoute = function (routeName) {
        if (routes[routeName])
            return routes[routeName];
        return routeName;
    };

    return Dispatcher;
});