﻿// todo this file is shared between the web and mobile site
define([
    'require',
    'jquery',
    'underscore'
], function (require, $, _) {
    'use strict';

    var suffixByEnvironment = {
        'designer': 'Design',
        'runtime': ''
    };

    function capitalize(string) {
        return string.charAt(0).toUpperCase() + string.slice(1);
    }

    function getLocationOfDrop(environment, moduleRoot, dropName) {
        var suffix = suffixByEnvironment[environment];
        if (suffix === undefined)
            throw new Error('Unable to create the drop: environment is not set.');

        return moduleRoot + '/' + capitalize(dropName) + '/' + dropName + suffix;
    };

    function DropFactory() { }

    DropFactory.prototype.createDrop = function (environment, moduleRoot, type, data, $element) {
        return $.Deferred(function (dfd) {
            require([getLocationOfDrop(environment, moduleRoot, type)], function (ctor) {
                var drop = new ctor(data, $element || $('<div/>'));
                drop.environment = environment;
                drop.type = type;
                drop.id = data.id;

                dfd.resolve(drop);
            });
        }).promise();
    };

    DropFactory.prototype.createDrops = function (environment, moduleRoot, type, arrayOfData) {
        return $.Deferred(function (dfd) {
            require([getLocationOfDrop(environment, moduleRoot, type)], function (ctor) {
                var drops = _.map(arrayOfData, function (data) {
                    var drop = new ctor(data, $('<div/>'));
                    drop.environment = environment;
                    drop.type = type;
                    drop.id = data.id;
                    return drop;
                });

                dfd.resolve(drops);
            });
        }).promise();
    };

    // todo clean this up
    DropFactory.prototype.restoreDrops = function (environment, moduleRoot, drops, $elementTemplate) {
        return $.Deferred(function (dfd) {
            drops = _.map(drops, function (drop) {
                var retVal = {
                    id: drop.id,
                    data: JSON.parse(drop.definition)
                };

                if (drop.dropRuntimeData)
                    retVal.data = $.extend(retVal.data, drop.dropRuntimeData);

                return retVal;
            });

            var dropLocations = _.chain(drops)
                .unique()
                .map(function (data) {
                    return getLocationOfDrop(environment, moduleRoot, data.data.type);
                })
                .value();

            require(dropLocations, function () {
                var dropConstructors = arguments;
                var restoredDrops = [];
                _.each(drops, function (drop) {
                    var constructorIndex = _.indexOf(dropLocations, getLocationOfDrop(environment, moduleRoot, drop.data.type));
                    var $element = $elementTemplate ? $elementTemplate.clone() : $('<div/>');
                    var restoredDrop = new dropConstructors[constructorIndex](drop.data, $element);
                    restoredDrop.id = drop.id;
                    restoredDrop.environment = environment;
                    restoredDrop.type = drop.data.type;
                    restoredDrops.push(restoredDrop);
                });

                dfd.resolve(restoredDrops);
            });
        }).promise();
    };

    DropFactory.prototype.restoreDrop = function (environment, moduleRoot, drop, $element) {
        return $.Deferred(function (dfd) {
            var data = JSON.parse(drop.definition);

            if (drop.dropRuntimeData)
                data = $.extend(data, drop.dropRuntimeData);

            require([getLocationOfDrop(environment, moduleRoot, data.type)], function (ctor) {
                var restoredDrop = new ctor(data, $element || $('<div/>'));
                restoredDrop.id = drop.id;
                restoredDrop.environment = environment;
                restoredDrop.type = data.type;
                dfd.resolve(restoredDrop);
            });
        }).promise();
    };

    return new DropFactory();
});
