﻿define([
    'jquery',
    'hammer',
    './url',
    'nico/cardRuntime',
    'nico/userSettings',
    'nico/dispatcher'
], function ($, Hammer, url, CardRuntime, userSettings, Dispatcher) {
    'use strict';

    var internalAppLinks = {
        'Profiel/actiefProgramma.html': 'programma.html',
        'Profiel/taal.html': 'taal.html'
    };

    var MenuRuntime = function ($button, $drawer) {
        this.$button = $button;
        this.$drawer = $drawer;
        this.$menuContent = $drawer.find(".contentPanel");

        var self = this;
        this.$button
            .click(function () {
                self.toggle();
            });

        Hammer(this.$drawer.get(0)).on('swipeleft', function () {
            self.hide();
        });
    };

    MenuRuntime.prototype.dispatcher = new Dispatcher();

    MenuRuntime.prototype.toggle = function () {
        this.$drawer.toggleClass('open');
    };

    MenuRuntime.prototype.hide = function () {
        this.$drawer.removeClass('open');
    };

    MenuRuntime.prototype.render = function (data) {
        var self = this;
        var $menuItems = _.map(data.menuItems, function (menuItem) {
            return self.createMenuItem(menuItem);
        });
        this.$menuContent.append($menuItems);

        this.$menuContent.append(this.createFixedItem(data.afmeldenTekst, 'menu-icon icon icon-power')
            .click(function () {
                self.logout();
            }));
    };

    MenuRuntime.prototype.logout = function () {
        this.dispatcher
            .post('invalidateToken')
            .always(function () {
                userSettings.delete();
                window.location.href = 'login.html';
            });
    };

    MenuRuntime.prototype.createMenuItem = function (menuItem) {
        var iconSpan = $('<span class="menu-icon icon">').addClass(menuItem.icon);
        var postSpan = $('<span class="icon icon-chevron-right" />)');

        var $div = $('<div />')
            .addClass('entry clickable')
            .text(menuItem.tekst)
            .prepend(iconSpan)
            .append(postSpan);

        if (menuItem.type === 'link') {
            $div.click(function () {
                if (internalAppLinks[menuItem.typeId])
                    window.location.href = internalAppLinks[menuItem.typeId];
                else
                    url.redirect(menuItem.typeId);
            });
        } else {
            var self = this;
            $div.click(function () {
                self.hide();

                if (!this.cardRuntime)
                    this.cardRuntime = new CardRuntime(this, menuItem.typeId, menuItem.cardKant);
                this.cardRuntime.openCard();
            });
        }

        return $div;
    };

    MenuRuntime.prototype.createFixedItem = function (title, icon) {
        return $('<div />')
            .addClass('entry clickable')
            .text(title)
            .prepend($('<span />').addClass(icon))
            .append($('<span class="icon icon-chevron-right" />'));
    };

    return MenuRuntime;
});