﻿define([
    'jquery',
    'underscore'
], function($, _) {

    var ModalWindow = function ($window) {
        this.id = $window.attr('id');
        this.$window = $window;
        this.$content = $window.find('.page');
        this.$body = $('body');

        this.deferred = null;

        var self = this;
        this.$window.click('click', function (e) {
            if (self.shouldCloseWindow(e)) {
                e.preventDefault();
                e.stopPropagation();
                self.close();
            }
        });
    };

    ModalWindow.prototype.shouldCloseWindow = function (e) {
        if (this.doNotAutoClose)
            return false;

        return $(e.target).is('.modal-window')
            || $(e.target).is('.modal-close')
            || $(e.target).is('.close-window');
    };

    ModalWindow.prototype.setContent = function ($content) {
        this.$content.append($content);
        return this;
    };

    ModalWindow.prototype.show = function (options) {
        options = options || {};
        this.doNotAutoClose = options.doNotAutoClose || false;
        this.$window.fadeIn();
        return this;
    };

    ModalWindow.prototype.promise = function () {
        this.deferred = $.Deferred();
        return this.deferred;
    };

    ModalWindow.prototype.close = function () {
        this.$window.fadeOut();

        if (this.deferred) {
            this.deferred.resolve();
            this.deferred = null;
        }
    };
    
    var loadedModals = {};
    
    return {
        getModal: function(idOfWindow) {
            if (!loadedModals[idOfWindow]) {
                var $window = $('#' + idOfWindow);
                loadedModals[idOfWindow] = new ModalWindow($window);
            }

            return loadedModals[idOfWindow];
        },
        create: function ($window) {
            return new ModalWindow($window);
        }
    };
})