// todo this file is shared between the web and mobile site
define([
    'underscore'
], function (_) {
    'use strict';

    function Observable() {
        this.eventRegistrations = {};
    }

    Observable.prototype.addListener = function (event, listener, optionalOwner) {
        if (!this.eventRegistrations[event])
            this.eventRegistrations[event] = [];
        this.eventRegistrations[event].push({
            listener: listener,
            owner: optionalOwner
        });
    };

    Observable.prototype.removeListener = function (event, owner) {
        var registrations = this.eventRegistrations[event];
        var registration = _.findWhere(registrations, { owner: owner });
        var index = _.indexOf(registrations, registration);
        registrations.splice(index, 1);
    };

    Observable.prototype.informListeners = function (event) {
        if (!this.eventRegistrations[event])
            return;

        var args = Array.prototype.slice.call(arguments, 1);
        _.forEach(this.eventRegistrations[event], function (registration) {
            registration.listener.apply(null, args);
        });
    };

    return {
        makeObservable: function (obj) {
            var observable = new Observable();
            obj.addListener = _.bind(observable.addListener, observable);
            obj.removeListener = _.bind(observable.removeListener, observable);
            obj.informListeners = _.bind(observable.informListeners, observable);
        }
    };
});