define([
    'nico/userSettings',
    'nico/url'
], function(userSettings, url) {

    var TokenMaster = function() {
        this.authentication = userSettings.get().authentication;
    };

    TokenMaster.prototype.hasToken = function() {
        return this.authentication != null;
    };

    TokenMaster.prototype.getToken = function() {
        if (!this.hasToken())
            throw new Error('No token available. Please request a new token first.');
        return this.authentication.token;
    };

    TokenMaster.prototype.setAuthentication = function (email, wachtwoord, token) {
        this.authentication = {
            authentication: {
                token: token,
                email: email,
                wachtwoord: wachtwoord
            }
        };
        userSettings.update(this.authentication);
    };

    TokenMaster.prototype.renew = function() {
        var promise = new $.Deferred();

        var self = this;
        $.ajax({
            url: url.getFullUrl('json'),
            type: 'POST',
            cache: false,
            data: JSON.stringify([{
                route: '/Home/RequestToken',
                data: {
                    email: this.authentication.email,
                    wachtwoord: this.authentication.wachtwoord
                }
            }]),
            dataType: 'json',
            contentType: 'application/json; charset=utf-8'
        })
        .done(function (results) {
            var result = results[0];

            if (result.wasSuccessful) {
                if (result.data.requiresActivation) {
                    window.location.href = 'login.html';
                } else {
                    self.authentication.token = result.data.token;

                    userSettings.update({
                        authentication: self.authentication
                    });

                    promise.resolve();
                }

            } else {
                self.authentication = null;

                userSettings.update({
                    authentication: null
                });

                promise.reject(result.message);
            }
        })
        .fail(function () {
            promise.reject('Er heeft zich een fout voorgedaan tijdens de authenticatie van de gebruiker.');
        });

        return promise;
    };

    return new TokenMaster();
});