define([
    'jquery',
    'iscroll',
    'underscore'
],function($, IScroll, _) {
    'use strict';

    function ViewPager($viewPager) {
        this.$viewPager = $viewPager;
        this.$slides = $viewPager.find('.slide');
        this.$scroller = $viewPager.find('.scroller');
        this.$tabs = [];
        this.scroller = null;

        this.wrapperWidth = 0;
        this.currentPage = 0;
    };

    ViewPager.prototype.render = function() {
        this.adjustPanelSizes();
        this.determineSizeOfPositionSlider();
        this.createScroller();
        this.setupNavigation();
        this.updateActiveTab();
        this.handleOrientationChange();
        this.handleEndOfScroll();
    };

    ViewPager.prototype.handleEndOfScroll = function() {
        var self = this;
        this.scroller.on('scrollEnd', function() {
            self.updateActiveTab();
        });
    };

    ViewPager.prototype.updateActiveTab = function() {
        var self = this;
        _.each(self.$tabs, function($tab) {
            if($tab.hasClass('active'))
                $tab.removeClass('active');
        });

        var index = this.scroller.currentPage.pageX;
        self.$tabs[index].addClass('active');
    };

    ViewPager.prototype.handleOrientationChange = function() {
        var self = this;
        window.addEventListener('resize', function() {
            self.adjustPanelSizes();

            _.defer(function() {
                self.scroller.refresh();
                self.scroller.goToPage(self.currentPage, 0, 0);
                self.updateActiveTab();
            });
        });
    };

    ViewPager.prototype.determineSizeOfPositionSlider = function() {
        this.$viewPager
            .find('.position')
            .css('width', (100/this.$slides.size()) + '%');
    };

    ViewPager.prototype.createScroller = function() {
        this.scroller = new IScroll(this.$viewPager.find('.wrapper').get(0), {
            scrollX: true,
            scrollY: false,
            bounce: false,
            momentum: false,
            directionLockThreshold: 0,

            eventPassthrough: true,
            preventDefault: false,

            snap: true,
            snapSpeed: 400,
            tap: true,
            indicators: {
                el: this.$viewPager.find('.slider').get(0),
                resize: false
            }
        });
    };

    ViewPager.prototype.setHeightOfViewContainer = function($element) {
        var wrapperHeight = this.$viewPager.find('.wrapper').height();
        $element.each(function() {
            var $this = $(this);
            var height = $this.children().outerHeight();
            if(height < wrapperHeight)
                height = wrapperHeight - ($this.children().outerHeight - $this.children().innerHeight);
            $this.css('height', height + 'px');
        });
    };

    ViewPager.prototype.setupNavigation = function() {
        if(this.$slides.length === 0)
            return;

        var $tabs = this.$viewPager.find('.tabs');

        var self = this;
        this.$tabs = _.map(this.$slides, function(slide, i) {
            var selector = $(slide).data('navigation-tab-selector');
            return $tabs
                .find(selector)
                .click(function() {
                    return self.openSlide(i);
                });
        });
    };

    ViewPager.prototype.openSlide = function(indexOfSlide) {
        this.scroller.goToPage(indexOfSlide, 0, 500);
    };

    ViewPager.prototype.adjustPanelSizes = function() {
        this.currentPage = 0;

        if (this.wrapperWidth > 0)
            this.currentPage = - Math.ceil(this.$scroller.position().left / this.wrapperWidth);

        this.wrapperWidth = this.$viewPager.find('.wrapper').width();

        this.$scroller.css('width', this.wrapperWidth * this.$slides.size());
        this.$slides.css('width', this.wrapperWidth - (this.$slides.outerWidth() - this.$slides.width()));
    };

    return ViewPager;
});