﻿// todo this file is shared between the web and mobile site
define([
    'jquery',
    'underscore',
    'nico/observable',
    'buildingBlocks/Widget/widget',
    'widgets/templateProvider'
], function ($, _, observable, Widget, templateProvider) {
    'use strict';

    var WidgetListLayoutRuntime = function (drops, $element, quizMaster, conditional) {
        observable.makeObservable(this);
        this.$element = $element;
        this.drops = drops;
        this.quizMaster = quizMaster;
        this.conditional = conditional;

        this.renderedPromise = $.Deferred();
    };

    WidgetListLayoutRuntime.prototype.promise = function () {
        return this.renderedPromise.promise();
    };

    WidgetListLayoutRuntime.prototype.render = function () {
        var self = this;

        var widgets = _.map(this.drops, function (data) {
            data.dropRuntimeData.quizMaster = self.quizMaster;
            data.dropRuntimeData.conditional = self.conditional;
            var $element = templateProvider.createInstanceOfTemplate(data.dropRuntimeData.templateName);
            var widget = new Widget(data.dropRuntimeData, $element);
            widget.parent = self;
            return widget;
        });

        var promises = [];
        _.each(widgets, function (widget) {
            widget.render();

            if (_.isFunction(widget.promise))
                promises.push(widget.promise());
        });

        if (promises.length === 0) {
            self.setWidgetWideIcon();
            self.renderedPromise.resolve();
        } else {
            $.when
                .apply($, promises)
                .done(function () {
                    self.setWidgetWideIcon();
                    self.renderedPromise.resolve();
                });
        }

        var $widgets = _.pluck(widgets, '$element');
        this.$element.append($widgets);
    };

    WidgetListLayoutRuntime.prototype.setWidgetWideIcon = function() {
        if (this.icon) {
            this.$element
                .find('.widgetListIcon')
                .addClass(this.icon);
        }
    };

    WidgetListLayoutRuntime.prototype.raiseEvent = function () {
        this.informListeners.apply(this, arguments);
    };

    WidgetListLayoutRuntime.prototype.close = function () {
        this.$element.empty();
    };

    WidgetListLayoutRuntime.prototype.getRootDrop = function () {
        return this;
    };

    return WidgetListLayoutRuntime;
});